#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_CHEMICALBONDINGRECORDER_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_CHEMICALBONDINGRECORDER_H

#include <string>
#include <filesystem>

#include "FileOwner.h"


namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Simulators
		{
			class ChemicalBondingRecorder :public ControlCenter::System::IO::FileOwner
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors and destructor

			public:
				ChemicalBondingRecorder();
				explicit ChemicalBondingRecorder(const std::filesystem::path& chemicalBondingRecorder);
				explicit ChemicalBondingRecorder(std::filesystem::path&& chemicalBondingRecorder);

				ChemicalBondingRecorder(ChemicalBondingRecorder&&) noexcept = default;
				ChemicalBondingRecorder& operator=(ChemicalBondingRecorder&&) noexcept = default;

				virtual ~ChemicalBondingRecorder() = default;

			// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				static std::string correctExtension() noexcept;
				static bool isCorrectExtension(const std::filesystem::path& filePath) noexcept;

				size_type currentTimeCounts() const noexcept;
				void resetCurrentTimeCounts(const size_type currentTimeCounts = 0) noexcept;

				void record(const size_type timeCountDistance, const double chemicalBondingRate, const double relaxedChemicalBondsRate, const double packingFraction);
				void clearAccumulativeRecords();
				void finishRecording();

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				void appendLogCash(const double chemicalBondingRate, const double relaxedChemicalBondsRate, const double packingFraction) const;
				void flushLogCash() const;

				void createPlotFile();

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				size_type m_currentTimeCounts;

				mutable size_type m_cashingCounts{ 0 };
				mutable std::string m_logCash{};

				static size_type m_maxCashingCounts;


			private:
				ChemicalBondingRecorder(const ChemicalBondingRecorder&) = delete;
				ChemicalBondingRecorder& operator=(const ChemicalBondingRecorder&) = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::string SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::correctExtension() noexcept
{
	return std::string{ ".txt" };
}

inline bool SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::isCorrectExtension(const std::filesystem::path& filePath) noexcept
{
	if (filePath.extension().string() == correctExtension())
		return true;
	else
		return false;
}

inline SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::size_type SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::currentTimeCounts() const noexcept
{
	return m_currentTimeCounts;
}

inline void SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::resetCurrentTimeCounts(const size_type currentTimeCounts) noexcept
{
	m_currentTimeCounts = currentTimeCounts;
}

inline void SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::record(const size_type timeCountDistance, const double chemicalBondingRate, const double relaxedChemicalBondsRate, const double packingFraction)
{
	m_currentTimeCounts += timeCountDistance;

	appendLogCash(chemicalBondingRate, relaxedChemicalBondsRate, packingFraction);
	++m_cashingCounts;

	if (m_maxCashingCounts <= m_cashingCounts)
		flushLogCash();
}

inline void SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::clearAccumulativeRecords()
{
	FileOwner::overCreateEmptyFile();
	m_currentTimeCounts = 0;
	m_cashingCounts = 0;
	m_logCash.clear();
}

inline void SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::finishRecording()
{
	flushLogCash();
	createPlotFile();
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline void SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::appendLogCash(const double chemicalBondingRate, const double relaxedChemicalBondsRate, const double packingFraction) const
{
	std::string additionalCash;
	{
		additionalCash += std::to_string(m_currentTimeCounts);
		additionalCash += "   ";
		additionalCash += std::to_string(chemicalBondingRate);
		additionalCash += "   ";
		additionalCash += std::to_string(relaxedChemicalBondsRate);
		additionalCash += "   ";
		additionalCash += std::to_string(packingFraction);
		additionalCash += "\n";
	}

	m_logCash += additionalCash;
}

inline void SolidStatePhysics::Crystallization::Simulators::ChemicalBondingRecorder::flushLogCash() const
{
	FileOwner::write(m_logCash);

	m_cashingCounts = 0;
	m_logCash.clear();
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_CHEMICALBONDINGRECORDER_H
