#ifndef SOLIDSTATEPHYSICS_MATERIALS_CHEMICALCONSTITUENTS_H
#define SOLIDSTATEPHYSICS_MATERIALS_CHEMICALCONSTITUENTS_H

#include "AtomicRoster.h"
#include "ChemicalComposition.h"


namespace SolidStatePhysics
{
	namespace Materials
	{
		class ChemicalConstituents :public AtomicRoster
		{
		public:
			using size_type = std::size_t;
			using iterator = std::map<size_type, size_type>::iterator;
			using const_iterator = std::map<size_type, size_type>::const_iterator;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			ChemicalConstituents() = default;
			virtual ~ChemicalConstituents() = default;

			ChemicalConstituents(const ChemicalConstituents&) = default;
			ChemicalConstituents(ChemicalConstituents&&) noexcept = default;
			ChemicalConstituents& operator=(const ChemicalConstituents&) = default;
			ChemicalConstituents& operator=(ChemicalConstituents&&) noexcept = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Utility

			std::string name() const;
			std::string getChemicalSystemName() const;
			ChemicalComposition toChemicalComposition() const noexcept;

		// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Utility

inline std::string SolidStatePhysics::Materials::ChemicalConstituents::name() const
{
	if (dictionary().empty())
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The function of \"name\" in the class of SolidStatePhysics::Materials::ChemicalConstituents.\n";
		errorMessage += "Message : This is invalid.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}

	else
	{
		std::string materialName;
		{
			for (const auto& numAndCount : dictionary())
			{
				materialName += SolidStatePhysics::ChemicalElements::PeriodicTable::replyElementSymbol(numAndCount.first);
				materialName += "_";
				materialName += std::to_string(numAndCount.second);
				materialName += "_";
			}

			materialName.pop_back();
		}

		return materialName;
	}
}

inline std::string SolidStatePhysics::Materials::ChemicalConstituents::getChemicalSystemName() const
{
	if (dictionary().empty())
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The function of \"name\" in the class of SolidStatePhysics::Materials::ChemicalConstituents.\n";
		errorMessage += "Message : This is invalid.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}

	else
	{
		std::string name;
		{
			for (const auto& numAndCount : dictionary())
			{
				name += SolidStatePhysics::ChemicalElements::PeriodicTable::replyElementSymbol(numAndCount.first);
				name += "-";
			}

			name.pop_back();
		}

		return name;
	}
}

// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_MATERIALS_CHEMICALCONSTITUENTS_H
