#ifndef CONTROLCENTER_SYSTEM_CLOCK_H
#define CONTROLCENTER_SYSTEM_CLOCK_H

#include <stdexcept>
#include <string>
#include <chrono>


namespace ControlCenter
{
	namespace System
	{
		class Clock final
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		public:
			Clock();
			~Clock() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			bool isRunningStopWatch() const noexcept;
			void startStopWatch() noexcept;
			void terminateStopWatch() noexcept;
			std::string tellElapsedTime();

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Static methods

			static std::string tellNowTime();

		// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			std::chrono::system_clock::time_point m_stopWatch;

			mutable bool m_isRunningStopWatch{ false };

		private:
			Clock(const Clock&) = delete;
			Clock(Clock&&) noexcept = delete;
			Clock& operator=(const Clock&) = delete;
			Clock& operator=(Clock&&) noexcept = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline bool ControlCenter::System::Clock::isRunningStopWatch() const noexcept
{
	return m_isRunningStopWatch;
}

inline void ControlCenter::System::Clock::startStopWatch() noexcept
{
	m_stopWatch = std::chrono::system_clock::now();
	m_isRunningStopWatch = true;
}

inline void ControlCenter::System::Clock::terminateStopWatch() noexcept
{
	m_stopWatch = std::chrono::system_clock::now();
	m_isRunningStopWatch = false;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CONTROLCENTER_SYSTEM_CLOCK_H
