#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_CONSTRAINTOWNERS_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_CONSTRAINTOWNERS_H

#include <string>
#include <stdexcept>
#include <memory>
#include <functional>

#include "IConstrainter.h"
#include "TranslatingAdaptor.h"


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Classes

namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class ConstraintOwner;

			class ChemicalBondOwner;
			class IonicConstraintOwner;
			class MetallicConstraintOwner;
			class PolyhedralConstraintOwner;
			class SecondNeighborConstraintOwner;
		}
	}
}

// Classes
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of ConstraintOwner

namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class InteractingAtom;
			class PeriodicLiquid;

			class ConstraintOwner
			{
				using IConstrainter = SolidStatePhysics::Crystallization::Calculators::IConstrainter;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operator

			public:
				explicit ConstraintOwner(InteractingAtom& ownerAtom);
				virtual ~ConstraintOwner() = default;

				ConstraintOwner(ConstraintOwner&&) noexcept;
				ConstraintOwner& operator=(ConstraintOwner&&) noexcept;

			// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				InteractingAtom& ownerAtom() const noexcept;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void join(const std::shared_ptr<IConstrainter>&);
				void join(std::shared_ptr<IConstrainter>&&);
				void rejoin(const std::shared_ptr<IConstrainter>&);
				void rejoin(std::shared_ptr<IConstrainter>&&);
				void disjoin() noexcept;
				bool isActive() const noexcept;

				virtual void registerOwingConstrainter(PeriodicLiquid& ownerPeriodicLiquid) const = 0;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Protected methods

			protected:
				const std::shared_ptr<IConstrainter>& constrainterPtr() const noexcept;

			// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::reference_wrapper<InteractingAtom> m_ownerAtom;
				std::shared_ptr<IConstrainter> m_constrainterPtr;


			private:
				ConstraintOwner() = delete;
				ConstraintOwner(const ConstraintOwner&) = delete;
				ConstraintOwner& operator=(const ConstraintOwner&) = delete;
			};
		}
	}
}

// The class of ConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of ChemicalBondOwner

namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class ChemicalBond;

			class ChemicalBondOwner : public ConstraintOwner
			{
			public:
				using Constrainter = ChemicalBond;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operator

			public:
				explicit ChemicalBondOwner(InteractingAtom& ownerAtom);
				virtual ~ChemicalBondOwner() = default;

				ChemicalBondOwner(ChemicalBondOwner&&) noexcept = default;
				ChemicalBondOwner& operator=(ChemicalBondOwner&&) noexcept = default;

			// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				const ChemicalBond& owingChemicalBond() const;
				ChemicalBond& owingChemicalBond();

				void registerOwingConstrainter(PeriodicLiquid&) const override;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				ChemicalBondOwner() = delete;
				ChemicalBondOwner(const ChemicalBondOwner&) = delete;
				ChemicalBondOwner& operator=(const ChemicalBondOwner&) = delete;
			};

			using ChemicalBondOwnerAdaptor = SolidStatePhysics::Crystallization::Calculators::TranslatingAdaptor<ChemicalBondOwner>;
		}
	}
}

// The class of ChemicalBondOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of IonicConstrainter

namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class IonicConstrainter;

			class IonicConstraintOwner : public ConstraintOwner
			{
			public:
				using Constrainter = IonicConstrainter;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operator

			public:
				explicit IonicConstraintOwner(InteractingAtom& ownerAtom);
				virtual ~IonicConstraintOwner() = default;

				IonicConstraintOwner(IonicConstraintOwner&&) noexcept = default;
				IonicConstraintOwner& operator=(IonicConstraintOwner&&) noexcept = default;

			// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				const IonicConstrainter& owingIonicConstrainter() const;
				IonicConstrainter& owingIonicConstrainter();

				void registerOwingConstrainter(PeriodicLiquid&) const override;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				IonicConstraintOwner() = delete;
				IonicConstraintOwner(const IonicConstraintOwner&) = delete;
				IonicConstraintOwner& operator=(const IonicConstraintOwner&) = delete;
			};

			using IonicConstraintOwnerAdaptor = SolidStatePhysics::Crystallization::Calculators::TranslatingAdaptor<IonicConstraintOwner>;
		}
	}
}

// The class of IonicConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of MetallicConstraintOwner

namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class MetallicConstrainter;

			class MetallicConstraintOwner : public ConstraintOwner
			{
			public:
				using Constrainter = MetallicConstrainter;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operator

			public:
				explicit MetallicConstraintOwner(InteractingAtom& ownerAtom);
				virtual ~MetallicConstraintOwner() = default;

				MetallicConstraintOwner(MetallicConstraintOwner&&) noexcept = default;
				MetallicConstraintOwner& operator=(MetallicConstraintOwner&&) noexcept = default;

			// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				const MetallicConstrainter& owingMetallicConstrainter() const;
				MetallicConstrainter& owingMetallicConstrainter();

				void registerOwingConstrainter(PeriodicLiquid&) const override;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				MetallicConstraintOwner() = delete;
				MetallicConstraintOwner(const MetallicConstraintOwner&) = delete;
				MetallicConstraintOwner& operator=(const MetallicConstraintOwner&) = delete;
			};

			using MetallicConstraintOwnerAdaptor = SolidStatePhysics::Crystallization::Calculators::TranslatingAdaptor<MetallicConstraintOwner>;
		}
	}
}

// The class of MetallicConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of PolyhedralConstraintOwner

namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class PolyhedralConstrainter;

			class PolyhedralConstraintOwner : public ConstraintOwner
			{
			public:
				using Constrainter = PolyhedralConstrainter;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operator

			public:
				explicit PolyhedralConstraintOwner(InteractingAtom& ownerAtom);
				virtual ~PolyhedralConstraintOwner() = default;

				PolyhedralConstraintOwner(PolyhedralConstraintOwner&&) noexcept = default;
				PolyhedralConstraintOwner& operator=(PolyhedralConstraintOwner&&) noexcept = default;

			// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				const PolyhedralConstrainter& owingPolyhedralConstrainter() const;
				PolyhedralConstrainter& owingPolyhedralConstrainter();

				void registerOwingConstrainter(PeriodicLiquid&) const override;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				PolyhedralConstraintOwner() = delete;
				PolyhedralConstraintOwner(const PolyhedralConstraintOwner&) = delete;
				PolyhedralConstraintOwner& operator=(const PolyhedralConstraintOwner&) = delete;
			};

			using PolyhedralConstraintOwnerAdaptor = SolidStatePhysics::Crystallization::Calculators::TranslatingAdaptor<PolyhedralConstraintOwner>;
		}
	}
}

// The class of PolyhedralConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of SecondNeighborConstraintOwner

namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class SecondNeighborConstrainter;

			class SecondNeighborConstraintOwner : public ConstraintOwner
			{
			public:
				using Constrainter = SecondNeighborConstrainter;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operator

			public:
				explicit SecondNeighborConstraintOwner(InteractingAtom& ownerAtom);
				virtual ~SecondNeighborConstraintOwner() = default;

				SecondNeighborConstraintOwner(SecondNeighborConstraintOwner&&) noexcept = default;
				SecondNeighborConstraintOwner& operator=(SecondNeighborConstraintOwner&&) noexcept = default;

			// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				const SecondNeighborConstrainter& owingSecondNeighborConstrainter() const;
				SecondNeighborConstrainter& owingSecondNeighborConstrainter();

				void registerOwingConstrainter(PeriodicLiquid&) const override;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				SecondNeighborConstraintOwner() = delete;
				SecondNeighborConstraintOwner(const SecondNeighborConstraintOwner&) = delete;
				SecondNeighborConstraintOwner& operator=(const SecondNeighborConstraintOwner&) = delete;
			};

			using SecondNeighborConstraintOwnerAdaptor = SolidStatePhysics::Crystallization::Calculators::TranslatingAdaptor<SecondNeighborConstraintOwner>;
		}
	}
}

// The class of SecondNeighborConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Methods for the class of ConstraintOwner

inline SolidStatePhysics::Crystallization::Components::InteractingAtom& SolidStatePhysics::Crystallization::Components::ConstraintOwner::ownerAtom() const noexcept
{
	return m_ownerAtom.get();
}

inline void SolidStatePhysics::Crystallization::Components::ConstraintOwner::join(const std::shared_ptr<IConstrainter>& constrainterPtr)
{
	if (m_constrainterPtr)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"join\" in the class of SolidStatePhysics::Crystallization::Components::ConstraintOwner.\n";
		errorMessage += "Message : This has already been active.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}

	else
		rejoin(constrainterPtr);
}

inline void SolidStatePhysics::Crystallization::Components::ConstraintOwner::join(std::shared_ptr<IConstrainter>&& constrainterPtr)
{
	if (m_constrainterPtr)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"join\" in the class of SolidStatePhysics::Crystallization::Components::ConstraintOwner.\n";
		errorMessage += "Message : This has already been active.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}

	else
		rejoin(std::move(constrainterPtr));
}

inline void SolidStatePhysics::Crystallization::Components::ConstraintOwner::rejoin(const std::shared_ptr<IConstrainter>& constrainterPtr)
{
	if (constrainterPtr)
		m_constrainterPtr = constrainterPtr;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"rejoin\" in the class of SolidStatePhysics::Crystallization::Components::ConstraintOwner.\n";
		errorMessage += "Message : \"constrainterPtr\" is invalid.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

inline void SolidStatePhysics::Crystallization::Components::ConstraintOwner::rejoin(std::shared_ptr<IConstrainter>&& constrainterPtr)
{
	if (constrainterPtr)
		m_constrainterPtr = std::move(constrainterPtr);

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"rejoin\" in the class of SolidStatePhysics::Crystallization::Components::ConstraintOwner.\n";
		errorMessage += "Message : \"constrainterPtr\" is invalid.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

inline void SolidStatePhysics::Crystallization::Components::ConstraintOwner::disjoin() noexcept
{
	m_constrainterPtr.reset();
}

inline bool SolidStatePhysics::Crystallization::Components::ConstraintOwner::isActive() const noexcept
{
	if (m_constrainterPtr)
		return true;
	else
		return false;
}

inline const std::shared_ptr<SolidStatePhysics::Crystallization::Calculators::IConstrainter>& SolidStatePhysics::Crystallization::Components::ConstraintOwner::constrainterPtr() const noexcept
{
	return m_constrainterPtr;
}

// Methods for the class of ConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_CONSTRAINTOWNERS_H
