#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_CRYSTALLIZATIONVARIABLES_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_CRYSTALLIZATIONVARIABLES_H

#include <stdexcept>
#include <string>


namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Simulators
		{
			class CrystallizationVariables
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				CrystallizationVariables();
				virtual ~CrystallizationVariables() = default;

				CrystallizationVariables(const CrystallizationVariables&) = default;
				CrystallizationVariables(CrystallizationVariables&&) noexcept = default;
				CrystallizationVariables& operator=(const CrystallizationVariables&) = default;
				CrystallizationVariables& operator=(CrystallizationVariables&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				size_type necessaryGeometricalOptimizing() const noexcept;
				size_type geometricalConstraintersLifetime() const noexcept;
				size_type linkedPolyhedraLifetime() const noexcept;
				size_type crystalLatticeLifetime() const noexcept;
				size_type thermalTransformationFrequency() const noexcept;
				double initialMaxDisplacement() const noexcept;
				double finalMaxDisplacement() const noexcept;

				bool needRenewGeometricalConstraints() const noexcept;
				bool needRenewLinkedPolyhedra() const noexcept;
				bool needNormalizeCrystalLattice() const noexcept;
				bool needThermalTransformation() const noexcept;

				void resetNecessaryGeometricalOptimizing(const size_type necessaryGeometricalOptimizing = 0) noexcept;
				void resetGeometricalConstraintsLifetime(const size_type geometricalConstraintsLifetime = 0) noexcept;
				void resetLinkedPolyhedraLifetime(const size_type linkedPolyhedraLifetime = 0) noexcept;
				void resetCrystalLatticeLifetime(const size_type crystalLatticeLifetime = 0) noexcept;
				void resetThermalTransformationFrequency(const size_type thermalTransformationFrequency = 0) noexcept;
				void resetMaxDisplacements(const double initialMaxDisplacement, const double finalMaxDisplacement);

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Utility

				void clear() noexcept;

			// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				size_type m_necessaryGeometricalOptimizing;
				size_type m_geometricalConstraintsLifetime;
				size_type m_linkedPolyhedraLifetime;
				size_type m_crystalLatticeLifetime;
				size_type m_thermalTransformationFrequency;

				double m_initialMaxDisplacement;
				double m_finalMaxDisplacement;
			};
		}
	}
}


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline std::size_t SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::necessaryGeometricalOptimizing() const noexcept
{
	return m_necessaryGeometricalOptimizing;
}

inline std::size_t SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::geometricalConstraintersLifetime() const noexcept
{
	return m_geometricalConstraintsLifetime;
}

inline std::size_t SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::linkedPolyhedraLifetime() const noexcept
{
	return m_linkedPolyhedraLifetime;
}

inline std::size_t SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::crystalLatticeLifetime() const noexcept
{
	return m_crystalLatticeLifetime;
}

inline std::size_t SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::thermalTransformationFrequency() const noexcept
{
	return m_thermalTransformationFrequency;
}

inline double SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::initialMaxDisplacement() const noexcept
{
	return m_initialMaxDisplacement;
}

inline double SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::finalMaxDisplacement() const noexcept
{
	return m_finalMaxDisplacement;
}

inline bool SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::needRenewGeometricalConstraints() const noexcept
{
	return static_cast<bool>(m_geometricalConstraintsLifetime);
}

inline bool SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::needRenewLinkedPolyhedra() const noexcept
{
	return static_cast<bool>(m_linkedPolyhedraLifetime);
}

inline bool SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::needNormalizeCrystalLattice() const noexcept
{
	return static_cast<bool>(m_crystalLatticeLifetime);
}

inline bool SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::needThermalTransformation() const noexcept
{
	return static_cast<bool>(m_thermalTransformationFrequency);
}

inline void SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::resetNecessaryGeometricalOptimizing(const size_type necessaryGeometricalOptimizing) noexcept
{
	m_necessaryGeometricalOptimizing = necessaryGeometricalOptimizing;
}

inline void SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::resetGeometricalConstraintsLifetime(const size_type geometricalConstraintsLifetime) noexcept
{
	m_geometricalConstraintsLifetime = geometricalConstraintsLifetime;
}

inline void SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::resetLinkedPolyhedraLifetime(const size_type linkedPolyhedraLifetime) noexcept
{
	m_linkedPolyhedraLifetime = linkedPolyhedraLifetime;
}

inline void SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::resetCrystalLatticeLifetime(const size_type crystalLatticeLifetime) noexcept
{
	m_crystalLatticeLifetime = crystalLatticeLifetime;
}

inline void SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::resetThermalTransformationFrequency(const size_type thermalTransformationFrequency) noexcept
{
	m_thermalTransformationFrequency = thermalTransformationFrequency;
}

inline void SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables::resetMaxDisplacements(const double initialMaxDisplacement, const double finalMaxDisplacement)
{
	if (initialMaxDisplacement < 0.0)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The function of \"resetMaxDisplacements\" in the class of SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables.\n";
		errorMessage += "Message : \"initialMaxDisplacement\" is less than zero.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}

	else
	{
		if (finalMaxDisplacement < 0.0)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The function of \"resetMaxDisplacements\" in the class of SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables.\n";
			errorMessage += "Message : \"finalMaxDisplacement\" is less than zero.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
		{
			if (initialMaxDisplacement < finalMaxDisplacement)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The function of \"resetMaxDisplacements\" in the class of SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables.\n";
				errorMessage += "Message : \"finalMaxDisplacement\" is more than \"initialMaxDisplacement\".\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}

			else
			{
				m_initialMaxDisplacement = initialMaxDisplacement;
				m_finalMaxDisplacement = finalMaxDisplacement;
			}
		}
	}
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_CRYSTALLIZATIONVARIABLES_H
