#ifndef SOLIDSTATEPHYSICS_UNITSYSTEM_ENERGYUNIT_H
#define SOLIDSTATEPHYSICS_UNITSYSTEM_ENERGYUNIT_H

#include <ratio>


namespace SolidStatePhysics
{
	namespace UnitSystem
	{
		namespace EnergyUnit
		{
			struct AtomicUnit
			{
				static constexpr double unitEnergyInSI() noexcept;
			};

			template <typename Magnitude = std::ratio<1>>
			struct Joule
			{
				static constexpr double unitEnergyInSI() noexcept;
			};

			template <typename Magnitude = std::ratio<1>>
			struct Electronvolt
			{
				static constexpr double unitEnergyInSI() noexcept;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constant expressions

inline constexpr double SolidStatePhysics::UnitSystem::EnergyUnit::AtomicUnit::unitEnergyInSI() noexcept
{
	return (4.3597447222071 * static_cast<double>(std::atto::num) / static_cast<double>(std::atto::den));
}

template <typename Magnitude>
inline constexpr double SolidStatePhysics::UnitSystem::EnergyUnit::Joule<Magnitude>::unitEnergyInSI() noexcept
{
	return (static_cast<double>(Magnitude::num) / static_cast<double>(Magnitude::den));
}

template <typename Magnitude>
inline constexpr double SolidStatePhysics::UnitSystem::EnergyUnit::Electronvolt<Magnitude>::unitEnergyInSI() noexcept
{
	constexpr double electronVolt = 0.1602176634 * (static_cast<double>(std::atto::num) / static_cast<double>(std::atto::den));
	return (electronVolt * static_cast<double>(Magnitude::num) / static_cast<double>(Magnitude::den));
}

// Constant expressions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_UNITSYSTEM_ENERGYUNIT_H
