#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_MONITORINGPLAN_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_MONITORINGPLAN_H

#include <stdexcept>


namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Simulators
		{
			class MonitoringPlan
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				MonitoringPlan();
				virtual ~MonitoringPlan() = default;

				MonitoringPlan(const MonitoringPlan&) = default;
				MonitoringPlan(MonitoringPlan&&) noexcept = default;
				MonitoringPlan& operator=(const MonitoringPlan&) = default;
				MonitoringPlan& operator=(MonitoringPlan&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				size_type mdRecordFrequency() const noexcept;
				size_type chemicalBondingRecordFrequency() const noexcept;
				bool needAllSuccessSampleData() const noexcept;
				bool needFailureData() const noexcept;
				bool needFinalSimulationsReportsPerStructure() const noexcept;

				bool needMdRecord() const noexcept;
				bool needChemicalBondingRecord() const noexcept;

				void resetMdRecordFrequency(const size_type mdRecordFrequency = 0) noexcept;
				void resetChemicalBondingRecordFrequency(const size_type chemicalBondingRecordFrequency = 0) noexcept;
				void resetAllSuccessSampleDataNecessity(const bool needAllSuccessSampleData) noexcept;
				void resetFailureDataNecessity(const bool needFailureData) noexcept;
				void resetFinalSimulationsReportsPerStructureNecessity(const bool finalSimulationsReportsPerStructureNecessity) noexcept;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Utility

				void clear() noexcept;

			// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				size_type m_mdRecordFrequency;
				size_type m_chemicalBondingRecordFrequency;
				bool m_needAllSuccessSampleData;
				bool m_needFailureData;
				bool m_needFinalSimulationsReportsPerStructure;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::size_t SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::mdRecordFrequency() const noexcept
{
	return m_mdRecordFrequency;
}

inline std::size_t SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::chemicalBondingRecordFrequency() const noexcept
{
	return m_chemicalBondingRecordFrequency;
}

inline bool SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::needAllSuccessSampleData() const noexcept
{
	return m_needAllSuccessSampleData;
}

inline bool SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::needFailureData() const noexcept
{
	return m_needFailureData;
}

inline bool SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::needFinalSimulationsReportsPerStructure() const noexcept
{
	return m_needFinalSimulationsReportsPerStructure;
}

inline bool SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::needMdRecord() const noexcept
{
	return static_cast<bool>(m_mdRecordFrequency);
}

inline bool SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::needChemicalBondingRecord() const noexcept
{
	return static_cast<bool>(m_chemicalBondingRecordFrequency);
}

inline void SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::resetMdRecordFrequency(const size_type mdRecordFrequency) noexcept
{
	m_mdRecordFrequency = mdRecordFrequency;
}

inline void SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::resetChemicalBondingRecordFrequency(const size_type chemicalBondingRecordFrequency) noexcept
{
	m_chemicalBondingRecordFrequency = chemicalBondingRecordFrequency;
}

inline void SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::resetAllSuccessSampleDataNecessity(const bool needAllSuccessSampleData) noexcept
{
	m_needAllSuccessSampleData = needAllSuccessSampleData;
}

inline void SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::resetFailureDataNecessity(const bool needFailureData) noexcept
{
	m_needFailureData = needFailureData;
}

inline void SolidStatePhysics::Crystallization::Simulators::MonitoringPlan::resetFinalSimulationsReportsPerStructureNecessity(const bool needFinalSimulationsReportsPerStructure) noexcept
{
	m_needFinalSimulationsReportsPerStructure = needFinalSimulationsReportsPerStructure;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_SIMULATORS_MONITORINGPLAN_H
