#ifndef NUMERICALANALYSIS_LINEARALGEBRA_NUMERICMATRIXTRAITS_H
#define NUMERICALANALYSIS_LINEARALGEBRA_NUMERICMATRIXTRAITS_H

#include <mkl_cblas.h>


namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		struct numeric_matrix_row_major_tag {};
		struct numeric_matrix_column_major_tag {};

		struct numeric_matrix_identity_tag {};
		struct numeric_matrix_transpose_tag {};
		struct numeric_matrix_conjugate_transpose_tag {};
		struct numeric_matrix_conjugate_tag {};
		struct numeric_matrix_invert_tag {};

		struct numeric_matrix_transform_traits
		{

		};
	}
}


namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		template <typename numeric_matrix_transform_tag>
		inline constexpr CBLAS_TRANSPOSE translateNumericMatrixTransposeTagToInt();

		template<>
		inline constexpr CBLAS_TRANSPOSE translateNumericMatrixTransposeTagToInt<numeric_matrix_identity_tag>()
		{
			return CblasNoTrans;
		}

		template<>
		inline constexpr CBLAS_TRANSPOSE translateNumericMatrixTransposeTagToInt<numeric_matrix_transpose_tag>()
		{
			return CblasTrans;
		}

		template<>
		inline constexpr CBLAS_TRANSPOSE translateNumericMatrixTransposeTagToInt<numeric_matrix_conjugate_transpose_tag>()
		{
			return CblasConjTrans;
		}


		template <typename numeric_matrix_transform_tag>
		inline constexpr char translateNumericMatrixTransposeTagToChar();

		template<>
		inline constexpr char translateNumericMatrixTransposeTagToChar<numeric_matrix_identity_tag>()
		{
			return 'N';
		}

		template<>
		inline constexpr char translateNumericMatrixTransposeTagToChar<numeric_matrix_transpose_tag>()
		{
			return 'T';
		}

		template<>
		inline constexpr char translateNumericMatrixTransposeTagToChar<numeric_matrix_conjugate_transpose_tag>()
		{
			return 'C';
		}
	}
}


#endif // !NUMERICALANALYSIS_LINEARALGEBRA_NUMERICMATRIXTRAITS_H
