#ifndef SOLIDSTATEPHYSICS_CHEMICALELEMENTS_PERIODICTABLE_H
#define SOLIDSTATEPHYSICS_CHEMICALELEMENTS_PERIODICTABLE_H

#include <string>
#include <stdexcept>


namespace SolidStatePhysics
{
	namespace ChemicalElements
	{
		class PeriodicTable
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			PeriodicTable() = default;
			virtual ~PeriodicTable() = default;

			PeriodicTable(const PeriodicTable&) = default;
			PeriodicTable(PeriodicTable&&) noexcept = default;
			PeriodicTable& operator=(const PeriodicTable&) = default;
			PeriodicTable& operator=(PeriodicTable&&) noexcept = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			static std::string replyElementSymbol(const std::size_t atomicNumber);
			static std::size_t replyAtomicNumber(const std::string& elementSymbol);
			static double replyAtomicWeight(const std::size_t atomicNumber);

			static bool isValidAtomicNumber(const std::size_t atomicNumber) noexcept;
			static bool isValidElementSymbol(const std::string& elementSymbol) noexcept;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			static void validateAtomicNumber(const std::size_t atomicNumber);
			static std::size_t translateElementSymbol(const std::string& elementSymbol) noexcept;

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::size_t SolidStatePhysics::ChemicalElements::PeriodicTable::replyAtomicNumber(const std::string& elementSymbol)
{
	std::size_t atomicNumber = translateElementSymbol(elementSymbol);
	{
		if (atomicNumber != 0)
			return atomicNumber;

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The function of \"replyAtomicNumber\" in the class of SolidStatePhysics::Materials::PeriodicTable.\n";
			errorMessage += "Message : \"elementSymbol\" is invalid.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}
	}
}

inline bool SolidStatePhysics::ChemicalElements::PeriodicTable::isValidAtomicNumber(const std::size_t atomicNumber) noexcept
{
	if (atomicNumber < 95)
	{
		if (atomicNumber != 0)
			return true;
		else
			return false;
	}

	else
		return false;
}

inline bool SolidStatePhysics::ChemicalElements::PeriodicTable::isValidElementSymbol(const std::string& elementSymbol) noexcept
{
	if (0 != translateElementSymbol(elementSymbol))
		return true;
	else
		return false;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline void SolidStatePhysics::ChemicalElements::PeriodicTable::validateAtomicNumber(const std::size_t atomicNumber)
{
	if (!(isValidAtomicNumber(atomicNumber)))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The function of \"validateAtomicNumber\" in the class of SolidStatePhysics::Materials::PeriodicTable.\n";
		errorMessage += "Message : \"atomicNumber\" is invalid.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CHEMICALELEMENTS_PERIODICTABLE_H
