#ifndef SOLIDSTATEPHYSICS_UNITSYSTEM_PHYSICALCONSTANTS_H
#define SOLIDSTATEPHYSICS_UNITSYSTEM_PHYSICALCONSTANTS_H

#include <ratio>

#include "VelocityUnit.h"
#include "EnergyUnit.h"
#include "TemperatureUnit.h"
#include "UnitConversion.h"


namespace SolidStatePhysics
{
	namespace UnitSystem
	{
		template <typename VelocityUnit = SolidStatePhysics::UnitSystem::VelocityUnit::AtomicUnit>
		constexpr double lightSpeed() noexcept
		{
			return speed_cast<VelocityUnit, SolidStatePhysics::UnitSystem::VelocityUnit::MetrePerSecond>{}(299792458.0);
		}

		template <typename EnergyUnit = SolidStatePhysics::UnitSystem::EnergyUnit::AtomicUnit>
		constexpr double boltzmannConstant() noexcept
		{
			constexpr double valueInSI = (13.80649 / (static_cast<double>(std::tera::num) * static_cast<double>(std::tera::num)));
			return energy_cast<EnergyUnit, SolidStatePhysics::UnitSystem::EnergyUnit::Joule<>>{}(valueInSI);
		}
	}
}


#endif // !SOLIDSTATEPHYSICS_UNITSYSTEM_PHYSICALCONSTANTS_H
