#ifndef CONTROLCENTER_ADMINISTERING_REHEATPERIODICSOLIDS_H
#define CONTROLCENTER_ADMINISTERING_REHEATPERIODICSOLIDS_H

#include <string>
#include <filesystem>
#include <vector>
#include <mutex>

#include "CrystallizationPlan.h"
#include "MonitoringPlan.h"


namespace ControlCenter
{
	namespace Administering
	{
		class ReheatPeriodicSolids
		{
			using size_type = std::size_t;
			using CrystallizationPlan = SolidStatePhysics::Crystallization::Simulators::CrystallizationPlan;
			using MonitoringPlan = SolidStatePhysics::Crystallization::Simulators::MonitoringPlan;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors and destructor

		public:
			explicit ReheatPeriodicSolids(const size_type threadIndex);
			ReheatPeriodicSolids(const size_type threadIndex, const CrystallizationPlan&, const MonitoringPlan&);
			ReheatPeriodicSolids(const size_type threadIndex, CrystallizationPlan&&, MonitoringPlan&&);

			virtual ~ReheatPeriodicSolids() = default;

			ReheatPeriodicSolids(const ReheatPeriodicSolids&) = default;
			ReheatPeriodicSolids(ReheatPeriodicSolids&&) noexcept = default;
			ReheatPeriodicSolids& operator=(const ReheatPeriodicSolids&) = default;
			ReheatPeriodicSolids& operator=(ReheatPeriodicSolids&&) noexcept = default;

		// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			const CrystallizationPlan& crystallizationPlan() const noexcept;
			const MonitoringPlan& monitoringPlan() const noexcept;

			void resetCrystallizationPlan(const CrystallizationPlan&);
			void resetCrystallizationPlan(CrystallizationPlan&&);
			void resetMonitoringPlan(const MonitoringPlan&);
			void resetMonitoringPlan(MonitoringPlan&&);

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			void operator()();

			static std::filesystem::path getReheatingChemicalConstituentsDirectoriesFilePath() noexcept;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			void operateCrystallizer(const std::filesystem::path& workDirectoryPath);

			std::filesystem::path assignWorkDirectory();
			void initializeWorkDirectory(const std::filesystem::path& workDirectoryPath);

			std::string getDirectoryName(std::filesystem::path directoryPath) const noexcept;

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			size_type m_threadIndex;
			CrystallizationPlan m_crystallizationPlan;
			MonitoringPlan m_monitoringPlan;

			static std::mutex m_reheatMutex;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const SolidStatePhysics::Crystallization::Simulators::CrystallizationPlan& ControlCenter::Administering::ReheatPeriodicSolids::crystallizationPlan() const noexcept
{
	return m_crystallizationPlan;
}

inline const SolidStatePhysics::Crystallization::Simulators::MonitoringPlan& ControlCenter::Administering::ReheatPeriodicSolids::monitoringPlan() const noexcept
{
	return m_monitoringPlan;
}

inline void ControlCenter::Administering::ReheatPeriodicSolids::resetCrystallizationPlan(const CrystallizationPlan& crystallizationPlan)
{
	m_crystallizationPlan = crystallizationPlan;
}

inline void ControlCenter::Administering::ReheatPeriodicSolids::resetCrystallizationPlan(CrystallizationPlan&& crystallizationPlan)
{
	m_crystallizationPlan = std::move(crystallizationPlan);
}

inline void ControlCenter::Administering::ReheatPeriodicSolids::resetMonitoringPlan(const MonitoringPlan& monitoringPlan)
{
	m_monitoringPlan = monitoringPlan;
}

inline void ControlCenter::Administering::ReheatPeriodicSolids::resetMonitoringPlan(MonitoringPlan&& monitoringPlan)
{
	m_monitoringPlan = std::move(monitoringPlan);
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline std::string ControlCenter::Administering::ReheatPeriodicSolids::getDirectoryName(std::filesystem::path directoryPath) const noexcept
{
	std::string name;
	{
		std::string directoryPathName = directoryPath.generic_string();

		if (directoryPathName.back() == '/')
		{
			directoryPathName.pop_back();
			directoryPath = std::filesystem::path{ directoryPathName };
		}

		name = directoryPath.filename().string();
	}

	return name;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CONTROLCENTER_ADMINISTERING_REHEATPERIODICSOLIDS_H
