#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_SECONDNEIGHBORCONSTRAINTER_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_SECONDNEIGHBORCONSTRAINTER_H

#include <string>
#include <stdexcept>
#include <functional>
#include <cmath>

#include "MinimumConstrainter.h"
#include "ConstraintOwners.h"


namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class SecondNeighborConstrainter :public SolidStatePhysics::Crystallization::Calculators::MinimumConstrainter
			{
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors and destructor

			public:
				SecondNeighborConstrainter(SecondNeighborConstraintOwner& realOwner, const SecondNeighborConstraintOwnerAdaptor& attachedOwner);
				SecondNeighborConstrainter(SecondNeighborConstraintOwner& realOwner, SecondNeighborConstraintOwnerAdaptor&& attachedOwner);
				explicit SecondNeighborConstrainter(const std::shared_ptr<SecondNeighborConstrainter>& twinPtr);

				virtual ~SecondNeighborConstrainter();

			// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Static methods

				static void create(SecondNeighborConstraintOwner& realOwner, const SecondNeighborConstraintOwnerAdaptor& attachedOwner);
				static void erase(std::shared_ptr<SecondNeighborConstrainter>&& secondNeighborConstrainterPtr);

			// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void renew() override;
				bool isFeasible(const double maxOverlapErrorRate) const noexcept;

				SecondNeighborConstraintOwner& realOwner() const noexcept;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::reference_wrapper<SecondNeighborConstraintOwner> m_realOwner;
				SecondNeighborConstraintOwnerAdaptor m_attachedOwner;


			private:
				SecondNeighborConstrainter() = delete;
				SecondNeighborConstrainter(const SecondNeighborConstrainter&) = delete;
				SecondNeighborConstrainter(SecondNeighborConstrainter&&) noexcept = delete;
				SecondNeighborConstrainter& operator=(const SecondNeighborConstrainter&) = delete;
				SecondNeighborConstrainter& operator=(SecondNeighborConstrainter&&) noexcept = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

inline void SolidStatePhysics::Crystallization::Components::SecondNeighborConstrainter::create(SecondNeighborConstraintOwner& realOwner, const SecondNeighborConstraintOwnerAdaptor& attachedOwner)
{
	InteratomicConnector::createInteratomicComponent(realOwner, attachedOwner);
}

inline void SolidStatePhysics::Crystallization::Components::SecondNeighborConstrainter::erase(std::shared_ptr<SecondNeighborConstrainter>&& secondNeighborConstrainterPtr)
{
	InteratomicConnector::eraseInteratomicComponent(std::move(secondNeighborConstrainterPtr));
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void SolidStatePhysics::Crystallization::Components::SecondNeighborConstrainter::renew()
{
	renewMinimumDistance(1.2 * (ownerAtom().maximumIonicRadius() + attachedAtom().entity().maximumIonicRadius()));
}

inline bool SolidStatePhysics::Crystallization::Components::SecondNeighborConstrainter::isFeasible(const double maxOverlapErrorRate) const noexcept
{
	double minBondLength = (1.0 - maxOverlapErrorRate) * minimumDistance();

	if (interatomicDistance() < minBondLength)
		return false;
	else
		return true;
}

inline SolidStatePhysics::Crystallization::Components::SecondNeighborConstraintOwner& SolidStatePhysics::Crystallization::Components::SecondNeighborConstrainter::realOwner() const noexcept
{
	return m_realOwner.get();
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_SECONDNEIGHBORCONSTRAINTER_H
