#ifndef NUMERICALANALYSIS_LINEARALGEBRA_SUPPORTFUNCTIONS_H
#define NUMERICALANALYSIS_LINEARALGEBRA_SUPPORTFUNCTIONS_H

#include "NumericVector.h"
#include "NumericMatrix.h"


namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		inline void swap(NumericVector& nva, NumericVector& nvb) noexcept
		{
			nva.swap(nvb);
		}
	}
}

namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		template <typename matrix_transform_tag>
		inline NumericMatrix::size_type getColumnSize(const NumericMatrix& nm);

		template <>
		inline NumericMatrix::size_type getColumnSize<numeric_matrix_identity_tag>(const NumericMatrix& nm)
		{
			return nm.columnSize();
		}

		template <>
		inline NumericMatrix::size_type getColumnSize<numeric_matrix_transpose_tag>(const NumericMatrix& nm)
		{
			return nm.rowSize();
		}

		template <>
		inline NumericMatrix::size_type getColumnSize<numeric_matrix_conjugate_transpose_tag>(const NumericMatrix& nm)
		{
			return nm.rowSize();
		}

		template <>
		inline NumericMatrix::size_type getColumnSize<numeric_matrix_conjugate_tag>(const NumericMatrix& nm)
		{
			return nm.columnSize();
		}

		template <>
		inline NumericMatrix::size_type getColumnSize<numeric_matrix_invert_tag>(const NumericMatrix& nm)
		{
			return nm.columnSize();
		}
	}
}

namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		template <typename matrix_transform_tag>
		inline NumericMatrix::size_type getRowSize(const NumericMatrix& nm);

		template <>
		inline NumericMatrix::size_type getRowSize<numeric_matrix_identity_tag>(const NumericMatrix& nm)
		{
			return nm.rowSize();
		}

		template <>
		inline NumericMatrix::size_type getRowSize<numeric_matrix_transpose_tag>(const NumericMatrix& nm)
		{
			return nm.columnSize();
		}

		template <>
		inline NumericMatrix::size_type getRowSize<numeric_matrix_conjugate_transpose_tag>(const NumericMatrix& nm)
		{
			return nm.columnSize();
		}

		template <>
		inline NumericMatrix::size_type getRowSize<numeric_matrix_conjugate_tag>(const NumericMatrix& nm)
		{
			return nm.rowSize();
		}

		template <>
		inline NumericMatrix::size_type getRowSize<numeric_matrix_invert_tag>(const NumericMatrix& nm)
		{
			return nm.rowSize();
		}
	}
}

namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		template <typename matrix_transform_tag>
		inline bool isMultipliable(const NumericMatrix& nm, const NumericVector& nv);

		template <>
		inline bool isMultipliable<numeric_matrix_identity_tag>(const NumericMatrix& nm, const NumericVector& nv)
		{
			if (nm.rowSize() == nv.size())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_tag>(const NumericMatrix& nm, const NumericVector& nv)
		{
			if (nm.rowSize() == nv.size())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_transpose_tag>(const NumericMatrix& nm, const NumericVector& nv)
		{
			if (nm.columnSize() == nv.size())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_transpose_tag>(const NumericMatrix& nm, const NumericVector& nv)
		{
			if (nm.columnSize() == nv.size())
			{
				return true;
			}

			else
			{
				return false;
			}
		}
	}
}

namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		template <typename first_matrix_transform_type, typename second_matrix_transform_type>
		inline bool isMultipliable(const NumericMatrix& nma, const NumericMatrix& nmb);

		template <>
		inline bool isMultipliable<numeric_matrix_identity_tag, numeric_matrix_identity_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_identity_tag, numeric_matrix_conjugate_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_tag, numeric_matrix_identity_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_tag, numeric_matrix_conjugate_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_identity_tag, numeric_matrix_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_identity_tag, numeric_matrix_conjugate_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_tag, numeric_matrix_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_tag, numeric_matrix_conjugate_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.rowSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_transpose_tag, numeric_matrix_identity_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_transpose_tag, numeric_matrix_conjugate_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_transpose_tag, numeric_matrix_identity_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_transpose_tag, numeric_matrix_conjugate_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.columnSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_transpose_tag, numeric_matrix_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_transpose_tag, numeric_matrix_conjugate_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_transpose_tag, numeric_matrix_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}

		template <>
		inline bool isMultipliable<numeric_matrix_conjugate_transpose_tag, numeric_matrix_conjugate_transpose_tag>(const NumericMatrix& nma, const NumericMatrix& nmb)
		{
			if (nma.columnSize() == nmb.rowSize())
			{
				return true;
			}

			else
			{
				return false;
			}
		}
	}
}



#endif // !NUMERICALANALYSIS_LINEARALGEBRA_SUPPORTFUNCTIONS_H
