#ifndef CONTROLCENTER_SYSTEM_THREADCONTROLLER_H
#define CONTROLCENTER_SYSTEM_THREADCONTROLLER_H

#include <string>
#include <stdexcept>


namespace ControlCenter
{
	namespace System
	{
		class ThreadController final
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		private:
			ThreadController() = default;
			~ThreadController() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

		public:
			static std::size_t defaultMaxThreading() noexcept;
			static std::size_t maxThreading() noexcept;
			static std::size_t mklThreading() noexcept;
			static std::size_t mklDomainThreading(const std::string& domainName);
			static std::size_t mklStriping() noexcept;
			static bool isMklDynamic() noexcept;

			static void resetMaxThreading(std::size_t maxThreading = 0) noexcept;
			static void resetMklThreading(std::size_t mklThreading = 0) noexcept;
			static void resetLocalMklThreading(std::size_t localMklThreading = 0) noexcept;
			static void resetMklDomainThreading(const std::string& domainName, std::size_t mklDomainThreading = 0);
			static void resetMklStriping(std::size_t mklStriping = 0) noexcept;
			static void setMklDynamic(const bool) noexcept;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			static int translateMklDomainName(const std::string& domainName);


		private:
			static std::size_t m_maxThreading;
			static std::size_t m_defaultMaxThreading;

		private:
			ThreadController(const ThreadController&) = delete;
			ThreadController(ThreadController&&) noexcept = delete;
			ThreadController& operator=(const ThreadController&) = delete;
			ThreadController& operator=(ThreadController&&) noexcept = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor, destructor, and static members

inline std::size_t ControlCenter::System::ThreadController::defaultMaxThreading() noexcept
{
	return m_defaultMaxThreading;
}

inline std::size_t ControlCenter::System::ThreadController::maxThreading() noexcept
{
	return m_maxThreading;
}

inline void ControlCenter::System::ThreadController::resetMaxThreading(std::size_t maxThreading) noexcept
{
	if (maxThreading == 0)
		m_maxThreading = m_defaultMaxThreading;
	else
		m_maxThreading = maxThreading;
}

// Access functions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CONTROLCENTER_SYSTEM_THREADCONTROLLER_H
