#ifndef SOLIDSTATEPHYSICS_UNITSYSTEM_TIMEUNIT_H
#define SOLIDSTATEPHYSICS_UNITSYSTEM_TIMEUNIT_H

#include <ratio>


namespace SolidStatePhysics
{
	namespace UnitSystem
	{
		namespace TimeUnit
		{
			struct AtomicUnit
			{
				static constexpr double unitTimeInSI() noexcept;
			};

			template <typename Magnitude = std::ratio<1>>
			struct Second
			{
				static constexpr double unitTimeInSI() noexcept;
			};


			using Attosecond = Second<std::atto>;
			using Femtosecond = Second<std::femto>;
			using Picosecond = Second<std::pico>;
			using Nanosecond = Second<std::nano>;
			using Microsecond = Second<std::micro>;
			using Millisecond = Second<std::milli>;
			using Minute = Second<std::ratio<60>>;
			using Hour = Second<std::ratio<3600>>;
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constant expressions

inline constexpr double SolidStatePhysics::UnitSystem::TimeUnit::AtomicUnit::unitTimeInSI() noexcept
{
	return ((0.024188843265857 * static_cast<double>(std::femto::num)) / static_cast<double>(std::femto::den));
}

template <typename Magnitude>
inline constexpr double SolidStatePhysics::UnitSystem::TimeUnit::Second<Magnitude>::unitTimeInSI() noexcept
{
	return (static_cast<double>(Magnitude::num) / static_cast<double>(Magnitude::den));
}

// Constant expressions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_UNITSYSTEM_TIMEUNIT_H
