#include "CrystalLattice.h"

using namespace SolidStatePhysics::Materials;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

CrystalLattice::CrystalLattice()
	: m_primitiveLatticeVectors(3, 3)
	, m_inversePrimitiveLatticeVectors(3, 3)
	, m_transformationTensor(3, 3)
{
	m_primitiveLatticeVectors(0, 0) = 1.0;
	m_primitiveLatticeVectors(1, 1) = 1.0;
	m_primitiveLatticeVectors(2, 2) = 1.0;

	m_inversePrimitiveLatticeVectors(0, 0) = 1.0;
	m_inversePrimitiveLatticeVectors(1, 1) = 1.0;
	m_inversePrimitiveLatticeVectors(2, 2) = 1.0;
}

CrystalLattice::CrystalLattice(const NumericMatrix& primitiveLatticeVectors)
	: m_primitiveLatticeVectors{ primitiveLatticeVectors }
	, m_inversePrimitiveLatticeVectors{}
	, m_transformationTensor(3, 3)
{
	if (m_primitiveLatticeVectors.columnSize() == 3 && m_primitiveLatticeVectors.rowSize() == 3)
		m_inversePrimitiveLatticeVectors = m_primitiveLatticeVectors.inverseMatrix();

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Materials::CrystalLattice.\n";
		errorMessage += "Message : The sizes of the column and/or row of \"primitiveLatticeVectors\" are not three.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

CrystalLattice::CrystalLattice(NumericMatrix&& primitiveLatticeVectors)
	: m_primitiveLatticeVectors{ std::move(primitiveLatticeVectors) }
	, m_inversePrimitiveLatticeVectors{}
	, m_transformationTensor(3, 3)
{
	if (m_primitiveLatticeVectors.columnSize() == 3 && m_primitiveLatticeVectors.rowSize() == 3)
		m_inversePrimitiveLatticeVectors = m_primitiveLatticeVectors.inverseMatrix();

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Materials::CrystalLattice.\n";
		errorMessage += "Message : The sizes of the column and/or row of \"primitiveLatticeVectors\" are not three.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void CrystalLattice::normalizeLatticeVector(const NumericVector& basisLatticeVector, NumericVector& optimizedLatticeVector) noexcept
{
	if (basisLatticeVector.normSquare() < NumericalAnalysis::LinearAlgebra::innerProduct(basisLatticeVector, optimizedLatticeVector))
	{
		while (true)
		{
			optimizedLatticeVector -= basisLatticeVector;

			if (NumericalAnalysis::LinearAlgebra::innerProduct(basisLatticeVector, optimizedLatticeVector) <= basisLatticeVector.normSquare())
				break;
		}
	}

	else
	{
		if ((NumericalAnalysis::LinearAlgebra::innerProduct(basisLatticeVector, optimizedLatticeVector) + basisLatticeVector.normSquare()) < 0.0)
		{
			while (true)
			{
				optimizedLatticeVector += basisLatticeVector;

				if (0.0 <= (NumericalAnalysis::LinearAlgebra::innerProduct(basisLatticeVector, optimizedLatticeVector) + basisLatticeVector.normSquare()))
					break;
			}
		}
	}
}

bool CrystalLattice::hasTooLongLatticeVector(const double maxLengthRatio) const noexcept
{
	CrystalLatticeParameters crystalLatticeParameters = getCrystalLatticeParameters();
	
	size_type tooLongRatioCount = 0;
	{
		if ((maxLengthRatio * crystalLatticeParameters.length.a) < crystalLatticeParameters.length.b)
			++tooLongRatioCount;
		if ((maxLengthRatio * crystalLatticeParameters.length.b) < crystalLatticeParameters.length.a)
			++tooLongRatioCount;
		if ((maxLengthRatio * crystalLatticeParameters.length.a) < crystalLatticeParameters.length.c)
			++tooLongRatioCount;
		if ((maxLengthRatio * crystalLatticeParameters.length.c) < crystalLatticeParameters.length.a)
			++tooLongRatioCount;
		if ((maxLengthRatio * crystalLatticeParameters.length.b) < crystalLatticeParameters.length.c)
			++tooLongRatioCount;
		if ((maxLengthRatio * crystalLatticeParameters.length.c) < crystalLatticeParameters.length.b)
			++tooLongRatioCount;
	}

	if (tooLongRatioCount < 2)
		return false;
	else
		return true;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
