#include "DirectoryManipulator.h"

using namespace ControlCenter::System::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void DirectoryManipulator::overDuplicateAllFiles(const std::filesystem::path& originalDirectoryPath, const std::filesystem::path& replicaDirectoryPath)
{
	validateDirectoryPath(originalDirectoryPath);
	{
		if (std::filesystem::exists(replicaDirectoryPath))
		{
			if (!(std::filesystem::is_directory(replicaDirectoryPath)))
			{
				std::filesystem::remove_all(replicaDirectoryPath);
				createDirectory(replicaDirectoryPath);
			}
		}

		else
			createDirectory(replicaDirectoryPath);
	}

	for (const std::filesystem::directory_entry& fileEntry : std::filesystem::directory_iterator(originalDirectoryPath))
	{
		std::filesystem::path originalFilePath = fileEntry.path();

		if (std::filesystem::is_regular_file(originalFilePath))
		{
			std::filesystem::path replicaFilePath = replicaDirectoryPath;
			replicaFilePath /= originalFilePath.filename();
			overDuplicateFile(originalFilePath, replicaFilePath);
		}

		else
			continue;
	}
}

bool DirectoryManipulator::tryDuplicateAllFiles(const std::filesystem::path& originalDirectoryPath, const std::filesystem::path& replicaDirectoryPath)
{
	validateDirectoryPath(originalDirectoryPath);

	if (std::filesystem::is_directory(replicaDirectoryPath))
	{
		for (const std::filesystem::directory_entry& fileEntry : std::filesystem::directory_iterator(originalDirectoryPath))
		{
			std::filesystem::path originalFilePath = fileEntry.path();

			if (std::filesystem::is_regular_file(originalFilePath))
			{
				std::filesystem::path replicaFilePath = replicaDirectoryPath;
				replicaFilePath /= originalFilePath.filename();
				tryDuplicateFile(originalFilePath, replicaFilePath);
			}

			else
				continue;
		}

		return true;
	}

	else
	{
		if (std::filesystem::exists(replicaDirectoryPath))
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"tryDuplicateAllFiles\" in the class of ControlCenter::Administering::IO::DirectoryManipulator.\n";
			errorMessage += "Message : ";
			{
				errorMessage += replicaDirectoryPath.generic_string();
				errorMessage += " is not a path to a directory.\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}

		else
			return false;
	}
}

void DirectoryManipulator::overTransmitAllFiles(const std::filesystem::path& oldDirectoryPath, const std::filesystem::path& newDirectoryPath)
{
	validateDirectoryPath(oldDirectoryPath);
	{
		if (std::filesystem::exists(newDirectoryPath))
		{
			std::filesystem::remove_all(newDirectoryPath);
			createDirectory(newDirectoryPath);
		}

		else
			createDirectory(newDirectoryPath);
	}

	for (const std::filesystem::directory_entry& fileEntry : std::filesystem::directory_iterator(oldDirectoryPath))
	{
		std::filesystem::path oldFilePath = fileEntry.path();

		if (std::filesystem::is_regular_file(oldFilePath))
		{
			std::filesystem::path newFilePath = newDirectoryPath;
			newFilePath /= oldFilePath.filename();
			overTransmitFile(oldFilePath, newFilePath);
		}

		else
			continue;
	}
}

bool DirectoryManipulator::tryTransmitAllFiles(const std::filesystem::path& oldDirectoryPath, const std::filesystem::path& newDirectoryPath)
{
	validateDirectoryPath(oldDirectoryPath);

	if (std::filesystem::is_directory(newDirectoryPath))
	{
		for (const std::filesystem::directory_entry& fileEntry : std::filesystem::directory_iterator(oldDirectoryPath))
		{
			std::filesystem::path oldFilePath = fileEntry.path();

			if (std::filesystem::is_regular_file(oldFilePath))
			{
				std::filesystem::path newFilePath = newDirectoryPath;
				newFilePath /= oldFilePath.filename();
				tryTransmitFile(oldFilePath, newFilePath);
			}

			else
				continue;
		}

		return true;
	}

	else
	{
		if (std::filesystem::exists(newDirectoryPath))
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"tryTransmitAllFiles\" in the class of ControlCenter::Administering::IO::DirectoryManipulator.\n";
			errorMessage += "Message : ";
			{
				errorMessage += newDirectoryPath.generic_string();
				errorMessage += " is not a path to a directory.\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}

		else
			return false;
	}
}

void DirectoryManipulator::overTransmitAllFilesRecursively(const std::filesystem::path& oldDirectoryPath, const std::filesystem::path& newDirectoryPath)
{
	validateDirectoryPath(oldDirectoryPath);
	validateParentDirectory(newDirectoryPath);

	if (std::filesystem::exists(newDirectoryPath))
		std::filesystem::remove_all(newDirectoryPath);

	std::filesystem::copy(oldDirectoryPath, newDirectoryPath, std::filesystem::copy_options::recursive);
	std::filesystem::remove_all(oldDirectoryPath);
}

void DirectoryManipulator::transmitAllFilesRecursively(const std::filesystem::path& oldDirectoryPath, const std::filesystem::path& newDirectoryPath)
{
	validateDirectoryPath(oldDirectoryPath);
	validateParentDirectory(newDirectoryPath);

	std::filesystem::copy(oldDirectoryPath, newDirectoryPath, std::filesystem::copy_options::recursive);
	std::filesystem::remove_all(oldDirectoryPath);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
