#include "FileOwner.h"

#include <fstream>

using namespace ControlCenter::System::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

FileOwner::FileOwner(const std::filesystem::path& filePath)
	: m_filePath{ filePath }
{
	if (!(m_filePath.has_filename()))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : filePath does not have a filename.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

FileOwner::FileOwner(std::filesystem::path&& filePath)
	: m_filePath{ std::move(filePath) }
{
	if (!(m_filePath.has_filename()))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : \"filePath\" does not have a filename.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void FileOwner::write(const std::string& message) const
{
	if (isValid())
	{
		std::ofstream outputFileStream;
		outputFileStream.open(m_filePath, std::ios_base::app);

		if (!outputFileStream)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"write\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
			errorMessage += "Message : Could not open the owning file of \"";
			{
				errorMessage += m_filePath.generic_string();
				errorMessage += "\".\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}

		else
		{
			outputFileStream << message;
			outputFileStream.close();
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"write\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The owning file path of \"";
		{
			errorMessage += filePath().generic_string();
			errorMessage += "\" is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

std::string FileOwner::read() const
{
	if (isValid())
	{
		std::string inputString;
		{
			std::ifstream inputFileStream;
			inputFileStream.open(m_filePath, std::ios_base::in);

			if (!inputFileStream)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"read\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
				errorMessage += "Message : Could not open the owning file of \"";
				{
					errorMessage += m_filePath.generic_string();
					errorMessage += "\".\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}

			else
			{
				for (std::string stringLine; std::getline(inputFileStream, stringLine);)
				{
					inputString += stringLine;
					inputString += "\n";
				}

				inputFileStream.close();
			}
		}

		return inputString;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"read\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The owning file path of \"";
		{
			errorMessage += filePath().generic_string();
			errorMessage += "\" is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

std::string FileOwner::readLine() const
{
	if (isValid())
	{
		std::ifstream inputFileStream;
		inputFileStream.open(m_filePath, std::ios_base::in);

		if (!inputFileStream)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"read\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
			errorMessage += "Message : Could not open the owning file of \"";
			{
				errorMessage += m_filePath.generic_string();
				errorMessage += "\".\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}

		else
		{
			std::string stringLine;
			std::getline(inputFileStream, stringLine);
			inputFileStream.close();

			return stringLine;
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"readLine\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The owning file path of \"";
		{
			errorMessage += filePath().generic_string();
			errorMessage += "\" is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

void FileOwner::clearLine() const
{
	if (isValid())
	{
		std::string inputString;
		{
			std::ifstream inputFileStream;
			inputFileStream.open(m_filePath, std::ios_base::in);

			if (!inputFileStream)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"read\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
				errorMessage += "Message : Could not open the owning file of \"";
				{
					errorMessage += m_filePath.generic_string();
					errorMessage += "\".\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}

			else
			{
				{
					std::string stringLine;
					std::getline(inputFileStream, stringLine);
				}

				for (std::string stringLine; std::getline(inputFileStream, stringLine);)
				{
					inputString += stringLine;
					inputString += "\n";
				}

				inputFileStream.close();
			}
		}

		overCreateFile(inputString);
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"read\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The owning file path of \"";
		{
			errorMessage += filePath().generic_string();
			errorMessage += "\" is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods on file system

void FileOwner::overCopy(const std::filesystem::path& replicaFilePath) const
{
	validateOwningFile();

	if (std::filesystem::exists(replicaFilePath))
		std::filesystem::remove_all(replicaFilePath);

	if (isValidFilePath(replicaFilePath))
	{
		if (!(std::filesystem::copy_file(m_filePath, replicaFilePath)))
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"overCopy\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
			errorMessage += "Message : Could not copy the file of ";
			{
				errorMessage += m_filePath.generic_string();
				errorMessage += " to the file of ";
				errorMessage += replicaFilePath.generic_string();
				errorMessage += ".\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"overCopy\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The \"replicaFilePath\" of ";
		{
			errorMessage += replicaFilePath.generic_string();
			errorMessage += " is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

bool FileOwner::tryCopy(const std::filesystem::path& replicaFilePath) const
{
	validateOwningFile();

	if (isValidFilePath(replicaFilePath))
	{
		if (std::filesystem::exists(replicaFilePath))
		{
			if (std::filesystem::is_regular_file(replicaFilePath))
				return false;

			else
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"tryCopy\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
				errorMessage += "Message : The replica file path of ";
				{
					errorMessage += replicaFilePath.generic_string();
					errorMessage += " is not a path to a regular file.\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}
		}

		else
		{
			if (std::filesystem::copy_file(m_filePath, replicaFilePath))
				return true;

			else
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"tryCopy\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
				errorMessage += "Message : Could not copy the file of ";
				{
					errorMessage += m_filePath.generic_string();
					errorMessage += " to the file of ";
					errorMessage += replicaFilePath.generic_string();
					errorMessage += ".\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"tryCopy\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The \"replicaFilePath\" of ";
		{
			errorMessage += replicaFilePath.generic_string();
			errorMessage += " is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

void FileOwner::overRename(const std::filesystem::path& newFilePath)
{
	validateOwningFile();

	if (isValidFilePath(newFilePath))
	{
		if (std::filesystem::exists(newFilePath))
		{
			std::filesystem::remove_all(newFilePath);
			std::filesystem::rename(m_filePath, newFilePath);
			m_filePath = newFilePath;
		}

		else
		{
			std::filesystem::rename(m_filePath, newFilePath);
			m_filePath = newFilePath;
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"overRename\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The new file path of ";
		{
			errorMessage += newFilePath.generic_string();
			errorMessage += " is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

bool FileOwner::tryRename(const std::filesystem::path& newFilePath)
{
	validateOwningFile();

	if (isValidFilePath(newFilePath))
	{
		if (std::filesystem::exists(newFilePath))
		{
			if (std::filesystem::is_regular_file(newFilePath))
				return false;

			else
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"tryRename\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
				errorMessage += "Message : The new file path of ";
				{
					errorMessage += newFilePath.generic_string();
					errorMessage += " is not a path to a regular file.\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}
		}

		else
		{
			std::filesystem::rename(m_filePath, newFilePath);
			m_filePath = newFilePath;
			return true;
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"tryRename\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The new file path of ";
		{
			errorMessage += newFilePath.generic_string();
			errorMessage += " is invalid.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

// Public methods on file system
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Protected methods

void FileOwner::overCreateFile(const std::string& message) const
{
	if (isOverCreatable())
	{
		remove();

		std::ofstream outputFileStream;
		outputFileStream.open(m_filePath, std::ios_base::out);

		if (!outputFileStream)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"overCreateEmptyFile\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
			errorMessage += "Message : Could not create the owning file of ";
			{
				errorMessage += m_filePath.generic_string();
				errorMessage += ".\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}

		else
		{
			if (message.empty())
				outputFileStream.close();

			else
			{
				outputFileStream << message;
				outputFileStream.close();
			}
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"overCreateEmptyFile\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
		errorMessage += "Message : The owning file of ";
		{
			errorMessage += m_filePath.generic_string();
			errorMessage += " is not creatable.\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

bool FileOwner::tryCreateFile(const std::string& message) const
{
	if (isTryCreatable())
	{
		std::ofstream outputFileStream;
		outputFileStream.open(m_filePath, std::ios_base::out);

		if (!outputFileStream)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"tryCreateFile\" in the class of ControlCenter::Administering::IO::FileOwner.\n";
			errorMessage += "Message : Could not try creating the owning file of ";
			{
				errorMessage += m_filePath.generic_string();
				errorMessage += ".\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}

		else
		{
			if (message.empty())
			{
				outputFileStream.close();
				return true;
			}

			else
			{
				outputFileStream << message;
				outputFileStream.close();
				return true;
			}
		}
	}

	else
		return false;
}

// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
