#include "IonicConstrainter.h"

using namespace SolidStatePhysics::Crystallization::Components;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors and destructor

double IonicConstrainter::m_formationRangeCoef{ 2.0 };


IonicConstrainter::IonicConstrainter(IonicConstraintOwner& realOwner, const IonicConstraintOwnerAdaptor& attachedOwner)
	: MinimumConstrainter{ realOwner.ownerAtom(), AtomicAdaptor(attachedOwner.entity().ownerAtom(), attachedOwner) }
	, m_realOwner{ std::ref(realOwner) }
	, m_attachedOwner{ attachedOwner }
{
}

IonicConstrainter::IonicConstrainter(IonicConstraintOwner& realOwner, IonicConstraintOwnerAdaptor&& attachedOwner)
	: MinimumConstrainter{ realOwner.ownerAtom(), AtomicAdaptor(attachedOwner.entity().ownerAtom(), attachedOwner) }
	, m_realOwner{ std::ref(realOwner) }
	, m_attachedOwner{ std::move(attachedOwner) }
{
}

IonicConstrainter::IonicConstrainter(const std::shared_ptr<IonicConstrainter>& twinPtr)
	: MinimumConstrainter{ std::static_pointer_cast<MinimumConstrainter>(twinPtr) }
	, m_realOwner{ std::ref(twinPtr->m_attachedOwner.entity()) }
	, m_attachedOwner{ twinPtr->m_realOwner, twinPtr->m_attachedOwner }
{
	m_attachedOwner.reverseLatticePoint();
}

IonicConstrainter::~IonicConstrainter()
{
	if (std::shared_ptr<IonicConstrainter> twinIonicConstrainterPtr = std::dynamic_pointer_cast<IonicConstrainter>(twinPtr()))
	{
		twinIonicConstrainterPtr->m_realOwner.get().disjoin();
		twinIonicConstrainterPtr->resetTwinPtr();
	}
}

// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

bool IonicConstrainter::shouldCreate(const std::shared_ptr<InteratomicConnector>& interatomicConnectorPtr) noexcept
{
	double radiiSum = (interatomicConnectorPtr->ownerAtom().maximumIonicRadius() + interatomicConnectorPtr->attachedAtom().entity().maximumIonicRadius());

	if (interatomicConnectorPtr->displacement().normSquare() < (m_formationRangeCoef * m_formationRangeCoef * radiiSum * radiiSum))
		return true;
	else
		return false;
}

bool IonicConstrainter::shouldCreate(const InteractingAtom& ownerAtom, const AtomicAdaptor& attachedAtom) noexcept
{
	NumericVector displacement = attachedAtom.entity().cartesianCoordinate() + attachedAtom.translationVector();
	displacement -= ownerAtom.cartesianCoordinate();
	double radiiSum = (ownerAtom.maximumIonicRadius() + attachedAtom.entity().maximumIonicRadius());

	if (displacement.normSquare() < (m_formationRangeCoef * m_formationRangeCoef * radiiSum * radiiSum))
		return true;
	else
		return false;
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
