#include "LogWriter.h"

#include <stdexcept>
#include <fstream>

#include "Clock.h"

using namespace ControlCenter::System::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor, destructor, and static member

std::filesystem::path LogWriter::m_logFilePath;

LogWriter::LogWriter()
{
	if (m_logFilePath.empty())
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The constructor of the class of ControlCenter::Administering::IO::LogWriter.\n";
		errorMessage += "Message : m_logFilePath is empty.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

LogWriter::LogWriter(const std::filesystem::path& logFilePath)
{
	if (m_logFilePath.empty())
	{
		if (logFilePath.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The constructor of the class of ControlCenter::Administering::IO::LogWriter.\n";
			errorMessage += "Message : logFilePath is empty.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
		{
			std::ofstream outputFileStream;
			outputFileStream.open(logFilePath, std::ios_base::out);

			if (!outputFileStream)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The constructor of the class of ControlCenter::Administering::IO::LogWriter.\n";
				errorMessage += "Message : Could not create the log file of ";
				{
					errorMessage += logFilePath.generic_string();
					errorMessage += ".\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}

			else
			{
				m_logFilePath = logFilePath;

				std::string headerMessage;
				headerMessage += "---------------------------------------------------------------------------------\n\n";
				headerMessage += "Creation date: ";
				{
					headerMessage += Clock::tellNowTime();
					headerMessage += "\n";
				}
				headerMessage += "\n---------------------------------------------------------------------------------\n\n";

				outputFileStream << headerMessage;
				outputFileStream.close();
			}
		}
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The constructor of the class of ControlCenter::Administering::IO::LogWriter.\n";
		errorMessage += "Message : m_logFilePath has already been initialized.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

// Constructor, destructor, and static member
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void LogWriter::report(const std::string& message)
{
	std::ofstream outputFileStream;
	outputFileStream.open(m_logFilePath, std::ios_base::app);

	if (!outputFileStream)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"report\" in the class of ControlCenter::Administering::IO::LogWriter.\n";
		errorMessage += "Message : Could not open the log file of ";
		{
			errorMessage += m_logFilePath.generic_string();
			errorMessage += ".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}

	else
	{
		outputFileStream << Clock::tellNowTime() << "\n";
		outputFileStream << message << "\n";
		outputFileStream.close();
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
