#include "MetallicConstrainter.h"

using namespace SolidStatePhysics::Crystallization::Components;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors and destructor

double MetallicConstrainter::m_formationRangeCoef{ 2.0 };


MetallicConstrainter::MetallicConstrainter(MetallicConstraintOwner& realOwner, const MetallicConstraintOwnerAdaptor& attachedOwner)
	: MinimumConstrainter{ realOwner.ownerAtom(), AtomicAdaptor(attachedOwner.entity().ownerAtom(), attachedOwner) }
	, m_realOwner{ std::ref(realOwner) }
	, m_attachedOwner{ attachedOwner }
{
}

MetallicConstrainter::MetallicConstrainter(MetallicConstraintOwner& realOwner, MetallicConstraintOwnerAdaptor&& attachedOwner)
	: MinimumConstrainter{ realOwner.ownerAtom(), AtomicAdaptor(attachedOwner.entity().ownerAtom(), attachedOwner) }
	, m_realOwner{ std::ref(realOwner) }
	, m_attachedOwner{ std::move(attachedOwner) }
{
}

MetallicConstrainter::MetallicConstrainter(const std::shared_ptr<MetallicConstrainter>& twinPtr)
	: MinimumConstrainter{ std::static_pointer_cast<MinimumConstrainter>(twinPtr) }
	, m_realOwner{ std::ref(twinPtr->m_attachedOwner.entity()) }
	, m_attachedOwner{ twinPtr->m_realOwner, twinPtr->m_attachedOwner }
{
	m_attachedOwner.reverseLatticePoint();
}

MetallicConstrainter::~MetallicConstrainter()
{
	if (std::shared_ptr<MetallicConstrainter> twinMetallicConstrainterPtr = std::dynamic_pointer_cast<MetallicConstrainter>(twinPtr()))
	{
		twinMetallicConstrainterPtr->m_realOwner.get().disjoin();
		twinMetallicConstrainterPtr->resetTwinPtr();
	}
}

// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

bool MetallicConstrainter::shouldCreate(const std::shared_ptr<InteratomicConnector>& interatomicConnectorPtr) noexcept
{
	double radiiSum = (interatomicConnectorPtr->ownerAtom().metallicRadius() + interatomicConnectorPtr->attachedAtom().entity().metallicRadius());

	if (interatomicConnectorPtr->displacement().normSquare() < (m_formationRangeCoef * m_formationRangeCoef * radiiSum * radiiSum))
		return true;
	else
		return false;
}

bool MetallicConstrainter::shouldCreate(const InteractingAtom& ownerAtom, const AtomicAdaptor& attachedAtom) noexcept
{
	NumericVector displacement = attachedAtom.entity().cartesianCoordinate() + attachedAtom.translationVector();
	displacement -= ownerAtom.cartesianCoordinate();
	double radiiSum = (ownerAtom.metallicRadius() + attachedAtom.entity().metallicRadius());

	if (displacement.normSquare() < (m_formationRangeCoef * m_formationRangeCoef * radiiSum * radiiSum))
		return true;
	else
		return false;
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
