#include "OrderForm.h"

#include "ThreadController.h"
#include "PeriodicTable.h"

using namespace ControlCenter::Window;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

OrderForm::OrderForm()
	: m_orderName{}
	, m_executionMode{ "GENERATE" }
	, m_maxThreading{ ControlCenter::System::ThreadController::defaultMaxThreading() }
	, m_mklThreading{ 1 }
	, m_outputDirectoryPath{}
	, m_workDirectoryPath{}
	, m_crystallizationPlan{}
	, m_monitoringPlan{}
	, m_requestedFeedstocks{}
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

std::string OrderForm::orderName() const noexcept
{
	if (m_orderName.empty())
	{
		if (m_requestedFeedstocks.isValid())
			return m_requestedFeedstocks.getRequestName();
		else
			return std::string{ "No-Name" };
	}

	else
		return m_orderName;
}

void OrderForm::clear() noexcept
{
	m_orderName.clear();
	m_executionMode = std::string{ "GENERATE" };
	m_maxThreading = ControlCenter::System::ThreadController::defaultMaxThreading();
	m_mklThreading = 1;
	m_outputDirectoryPath.clear();
	m_workDirectoryPath.clear();

	m_crystallizationPlan.clear();
	m_monitoringPlan.clear();
	m_requestedFeedstocks.clear();
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void OrderForm::resetOrderName(const std::string& orderName) noexcept
{
	if (orderName.empty())
	{
		if (m_requestedFeedstocks.isValid())
			m_orderName = m_requestedFeedstocks.getRequestName();
		else
			m_orderName.clear();
	}

	else
		m_orderName = orderName;
}

void OrderForm::resetMaxThreading(const std::size_t maxThreading)
{
	if (maxThreading == 0)
		m_maxThreading = ControlCenter::System::ThreadController::defaultMaxThreading();

	else
	{
		if (ControlCenter::System::ThreadController::defaultMaxThreading() < maxThreading)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The member function of \"resetMaxThreading\" in the class of ControlCenter::Window::OrderForm.\n";
			errorMessage += "Message : \"maxThreading\" is more than the default maximum theading.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
			m_maxThreading = maxThreading;
	}
}

void OrderForm::resetMklThreading(const std::size_t mklThreading)
{
	if (mklThreading == 0)
		m_mklThreading = 1;

	else
	{
		if (ControlCenter::System::ThreadController::defaultMaxThreading() < mklThreading)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The member function of \"resetMaxThreading\" in the class of ControlCenter::Window::OrderForm.\n";
			errorMessage += "Message : \"mklThreading\" is more than the default maximum theading.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
			m_mklThreading = mklThreading;
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
