#include "SystemPathsRepository.h"

#include "DirectoryManipulator.h"
#include "MpiController.h"

using namespace ControlCenter::System::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor, destructor, and static members

std::filesystem::path SystemPathsRepository::m_inputDirectoryPath;
std::filesystem::path SystemPathsRepository::m_outputDirectoryPath;
std::filesystem::path SystemPathsRepository::m_workDirectoryPath;

// Constructor, destructor and static members
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

void SystemPathsRepository::resetInputDirectoryPath(const std::filesystem::path& inputDirectoryPath)
{
	if (m_inputDirectoryPath.empty())
	{
		if (inputDirectoryPath.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The member function of \"resetInputDirectoryPath\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
			errorMessage += "Message : \"inputDirectoryPath\" is empty.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
			m_inputDirectoryPath = inputDirectoryPath;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"resetInputDirectoryPath\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
		errorMessage += "Message : \"m_inputDirectoryPath\" has already been initialized.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

void SystemPathsRepository::resetOutputDirectoryPath(const std::filesystem::path& outputDirectoryPath)
{
	if (m_outputDirectoryPath.empty())
	{
		if (outputDirectoryPath.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The member function of \"setOutputDirectoryPath\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
			errorMessage += "Message : \"outputDirectoryPath\" is empty.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
			m_outputDirectoryPath = outputDirectoryPath;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"setOutputDirectoryPath\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
		errorMessage += "Message : \"m_outputDirectoryPath\" has already been initialized.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

void SystemPathsRepository::resetWorkDirectoryPath(const std::filesystem::path& workDirectoryPath)
{
	if (m_workDirectoryPath.empty())
	{
		if (workDirectoryPath.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The member function of \"setWorkDirectoryPath\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
			errorMessage += "Message : \"workDirectoryPath\" is empty.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
			m_workDirectoryPath = workDirectoryPath;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"setWorkDirectoryPath\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
		errorMessage += "Message : \"m_workDirectoryPath\" has already been initialized.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void SystemPathsRepository::tryInitializeOutputDirectory()
{
	if (m_outputDirectoryPath.empty())
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"tryInitializeOutputDirectory\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
		errorMessage += "Message : \"m_outputDirectoryPath\" is empty.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}

	else
	{
		if (MpiController::isLeadingMpi())
			DirectoryManipulator::tryInitializeDirectory(m_outputDirectoryPath);
	}
}

void SystemPathsRepository::tryInitializeWorkDirectory()
{
	if (m_workDirectoryPath.empty())
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"tryInitializeWorkDirectory\" in the class of ControlCenter::Administering::IO::SystemPathsRepository.\n";
		errorMessage += "Message : \"m_workDirectoryPath\" is empty.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}

	else
	{
		if (MpiController::isLeadingMpi())
			DirectoryManipulator::tryInitializeDirectory(m_workDirectoryPath);
	}
}

void SystemPathsRepository::transportAllDataFromWorkToOutputDirectory()
{
	if (MpiController::isLeadingMpi())
		DirectoryManipulator::transmitAllFilesRecursively(m_workDirectoryPath, m_outputDirectoryPath);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
