#include "ThreadController.h"

#include <mkl_service.h>

using namespace ControlCenter::System;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor, destructor, and static members

std::size_t ThreadController::m_maxThreading = MKL_Get_Max_Threads();
std::size_t ThreadController::m_defaultMaxThreading = MKL_Get_Max_Threads();;

// Constructor, destructor, and static members
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::size_t ThreadController::mklThreading() noexcept
{
	return static_cast<std::size_t>(MKL_Get_Max_Threads());
}

std::size_t ThreadController::mklDomainThreading(const std::string& domainName)
{
	return static_cast<std::size_t>(MKL_Domain_Get_Max_Threads(translateMklDomainName(domainName)));
}

std::size_t ThreadController::mklStriping() noexcept
{
	return static_cast<std::size_t>(MKL_Get_Num_Stripes());
}

bool ThreadController::isMklDynamic() noexcept
{
	if (0 == MKL_Get_Dynamic())
		return false;
	else
		return true;
}

void ThreadController::resetMklThreading(std::size_t mklThreading) noexcept
{
	if (mklThreading == 0)
		MKL_Set_Num_Threads(static_cast<int>(m_maxThreading));
	else
		MKL_Set_Num_Threads(static_cast<int>(mklThreading));
}

void ThreadController::resetLocalMklThreading(std::size_t localMklThreading) noexcept
{
	MKL_Set_Num_Threads_Local(static_cast<int>(localMklThreading));
}

void ThreadController::resetMklDomainThreading(const std::string& domainName, std::size_t mklDomainThreading)
{
	if (MKL_Domain_Set_Num_Threads(static_cast<int>(mklDomainThreading), translateMklDomainName(domainName)) == 0)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"resetMklDomainThreading\" in the class of ControlCenter::Administering::ThreadController.\n";
		errorMessage += "Message : The function of \"MKL_Domain_Set_Num_Threads\" indicates a failure.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

void ThreadController::resetMklStriping(std::size_t mklStriping) noexcept
{
	MKL_Set_Num_Stripes(static_cast<int>(mklStriping));
}

void ThreadController::setMklDynamic(const bool requestDynamic) noexcept
{
	if (requestDynamic)
		MKL_Set_Dynamic(1);
	else
		MKL_Set_Dynamic(0);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

int ThreadController::translateMklDomainName(const std::string& domainName)
{
	if (domainName == "ALL" || domainName == "All" || domainName == "all")
		return MKL_DOMAIN_ALL;

	else
	{
		if (domainName == "BLAS" || domainName == "CBLAS")
			return MKL_DOMAIN_BLAS;

		else
		{
			if (domainName == "FFT")
				return MKL_DOMAIN_FFT;

			else
			{
				if (domainName == "VML")
					return MKL_DOMAIN_VML;

				else
				{
					if (domainName == "LAPACK" || domainName == "LAPACKE")
						return MKL_DOMAIN_LAPACK;

					else
					{
						std::string errorMessage = "------------------------------------------------------\n\n";
						errorMessage += "Error   : Invalid argument\n";
						errorMessage += "Location: The member function of \"translateMklDomainName\" in the class of ControlCenter::Administering::ThreadController.\n";
						errorMessage += "Message : domainName is nonsense.\n\n";
						errorMessage += "------------------------------------------------------\n\n";
						throw std::invalid_argument(errorMessage);
					}
				}
			}
		}
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
